<?php

namespace Drupal\users_export\Form;

use AKlump\LoftDataGrids\ExportData;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\users_export\UsersExport;
use Drupal\users_export\UserExportLoftDataGrids;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides UsersExport Form.
 */
class UsersExportForm extends ConfigFormBase {

  /**
   * Returns the date.formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  private $dateFormatter;

  /**
   * Returns the entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  private $dateFormatStorage;

  /**
   * Returns the users_export.loft_data_grids service.
   *
   * @var \Drupal\users_export\UserExportLoftDataGrids
   */
  private $loftDataGrids;

  /**
   * Returns the users_export.core service.
   *
   * @var \Drupal\users_export\UsersExport
   */
  private $core;

  /**
   * Constructs a UsersExportForm form.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   Provides an interface defining a date formatter.
   * @param \Drupal\Core\Entity\EntityStorageInterface $date_format_storage
   *   Defines the interface for entity storage classes.
   * @param \Drupal\users_export\UserExportLoftDataGrids $loft_data_grids
   *   Manager class to get available exporters.
   * @param \Drupal\users_export\UsersExport $core
   *   Base class to export users.
   */
  public function __construct(ConfigFactoryInterface $config_factory, DateFormatterInterface $date_formatter, EntityStorageInterface $date_format_storage, UserExportLoftDataGrids $loft_data_grids, UsersExport $core) {
    parent::__construct($config_factory);

    $this->dateFormatter = $date_formatter;
    $this->dateFormatStorage = $date_format_storage;
    $this->loftDataGrids = $loft_data_grids;
    $this->core = $core;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('date.formatter'),
      $container->get('entity_type.manager')->getStorage('date_format'),
      $container->get('users_export.loft_data_grids'),
      $container->get('users_export.core')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'users_export_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'users_export.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = $this->config('users_export.settings');

    foreach (Element::children($form) as $variable) {
      $config->set($variable, $form_state->getValue($form[$variable]['#parents']));
    }
    $config->save();

    if (method_exists($this, '_submitForm')) {
      $this->_submitForm($form, $form_state);
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * Form constructor for the users export form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $settings = $this->configFactory->get('users_export.settings');
    $options = $this->loftDataGrids->getExporterOptions(TRUE, FALSE, FALSE);

    // This does NOT work for library_load()
    $form['#attached']['library'][] = 'users_export/core';

    $exporters = $this->loftDataGrids->getExporters();
    $form_state->set('exporters', $exporters);

    $js_settings = [];
    foreach ($exporters as $exporter) {
      $js_settings[$exporter['id']] = array_intersect_key($exporter, array_flip(['extension']));
    }
    $form['#attached']['drupalSettings']['usersExport'] = $js_settings;

    $type = '';
    $class = $settings->get('users_export_type');
    if (isset($exporters[$class])) {
      $type = $exporters[$class]['extension'];
    }
    $form['users_export_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Export file format'),
      '#default_value' => $class ?? '',
      '#options' => $options,
    ];

    $default = $settings->get('users_export_filename');
    if (empty($default)) {
      if (!($name = $this->configFactory->get('system.site')->get('name'))) {
        $name = 'users_export';
      }
      $default = strtolower(preg_replace('/\W+/', '_', $name) . '_users');
    }
    $form['users_export_filename'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Filename to save as'),
      '#default_value' => $default,
      '#required' => TRUE,
      '#field_suffix'  => $type,
    ];

    $form['users_export_test_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Preview mode (Enable to limit the export to only the first 10 users to check formatting.)'),
      '#default_value' => $settings->get('users_export_test_mode') ?? '',
    ];

    $form['basic_filters'] = [
      '#type'  => 'details',
      '#title' => $this->t('Basic Filters'),
      '#open'  => FALSE,
    ];

    $form['advanced'] = [
      '#type'  => 'details',
      '#title' => $this->t('Advanced Settings'),
      '#open'  => FALSE,
    ];

    $form['basic_filters']['users_export_with_access'] = [
      '#type' => 'select',
      '#title' => $this->t('Login Frequency'),
      '#default_value' => $settings->get('users_export_with_access') ?? 2,
      '#options' => [
        2 => $this->t('All Users'),
        1 => $this->t('Get users who have logged in at least one time'),
        0 => $this->t('Get users who have never logged in'),
      ],
    ];

    // Add field to add 'blocked' users to the export as well.
    $form['basic_filters']['users_export_user_status'] = [
      '#type' => 'select',
      '#title' => $this->t('Export users with status'),
      '#default_value' => $settings->get('users_export_user_status') ?? 2,
      '#options' => [
        1 => $this->t('Active'),
        0 => $this->t('Blocked'),
        2 => $this->t('Both active and blocked'),
      ],
    ];

    // Option to Filter By Role. (Optional).
    $form['basic_filters']['users_export_filter_by_role'] = [
      '#type' => 'radios',
      '#title' => $this->t('Filter by role?'),
      '#default_value' => $settings->get('users_export_filter_by_role') ?? 0,
      '#options' => [
        0 => $this->t('No'),
        1 => $this->t('Yes'),
      ],
    ];

    $roles_options = [];
    foreach (user_roles(TRUE) as $role) {
      $roles_options[$role->id()] = $role->label();
    }

    $form['basic_filters']['users_export_user_role'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Export users with role'),
      '#default_value' => $settings->get('users_export_with_role') ?? [],
      '#options' => $roles_options,
      '#states' => [
        'invisible' => [
          ':input[name="users_export_filter_by_role"]' => ['value' => 0],
        ],
      ],
    ];

    $form['basic_filters']['users_export_order'] = [
      '#type' => 'select',
      '#title' => $this->t('Order of results'),
      '#default_value' => $settings->get('users_export_order') ?? 0,
      '#options' => [
        0 => $this->t('User ID'),
        1 => $this->t('Username A-Z'),
        2 => $this->t('E-mail A-Z'),
      ],
    ];

    $time = new DrupalDateTime();
    $format_types = $this->dateFormatStorage->loadMultiple();
    $options = [];
    foreach ($format_types as $type => $type_info) {
      $format = $this->dateFormatter->format($time->format('U'), $type);
      $options[$type_info->getPattern()] = $type_info->label() . ' (' . $format . ')';
    }

    // Now add in those we think most users will want to use.
    $options += [
      'Y-m-d' => 'Excel Date (' . date('Y-m-d') . ')',
      'Y-m-d H:i:s' => 'Excel Date & Time (' . date('Y-m-d H:i:s') . ')',
      'Y-m-d\TH:i:s\Z' => 'Datetime (' . date('Y-m-d\TH:i:s\Z') . ')',
      'U' => 'Unix Timestamp (' . date('U') . ')',
      \DateTime::ISO8601 => 'ISO 8601 (' . date(\DateTime::ISO8601) . ')',
      'Ymd\THis' => 'iCal (' . date('Ymd\THis') . ')',
    ];

    $form['advanced']['users_export_date_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Date format'),
      '#default_value' => $settings->get('users_export_date_format') ?? '',
      '#options' => $options,
    ];

    $options = [-1 => $this->t('- Default -')];
    foreach (range(128, 2048, 32) as $value) {
      $options[$value . 'M'] = format_size($value * 1024 * 1024);
    }
    $form['advanced']['users_export_memory_limit'] = [
      '#type' => 'select',
      '#title' => $this->t('Memory Limit', [
        '@size' => format_size($settings->get('users_export_last_export_memory')),
      ]),
      '#description' => $this->t('If you have many users you may need to set this value higher so you the web server does not run out of memory processing the export. <strong>Depending upon your server configuration, this may or may not have any effect!</strong> For more information refer to <a href="http://php.net/manual/en/function.ini-set.php" target="blank">http://php.net/manual/en/function.ini-set.php</a>.'),
      '#default_value' => $settings->get('users_export_memory_limit') ?? '-1',
      '#options' => $options,
    ];

    $options = [-1 => $this->t('- Default -')];
    foreach (range(30, 1800, 30) as $value) {
      $options[$value] = $this->dateFormatter->formatInterval($value);
    }
    $form['advanced']['users_export_max_execution'] = [
      '#type' => 'select',
      '#title' => $this->t('Maximum Execution Time', [
        '@time' => $this->dateFormatter->formatInterval($settings->get('users_export_last_export_time')),
      ]),
      '#description' => $this->t('If you have many users you may need to set this value higher so you the web server does not timeout. <strong>Depending upon your server configuration, this may or may not have any effect!</strong> For more information refer to <a href="http://php.net/manual/en/function.set-time-limit.php" target="blank">http://php.net/manual/en/function.set-time-limit.php</a>.'),
      '#default_value' => $settings->get('users_export_max_execution') ?? '',
      '#options' => $options,
    ];

    $form = parent::buildForm($form, $form_state);
    $form['actions']['submit']['#value'] = $this->t('Download File');

    return $form;
  }

  /**
   * Manage form submition.
   *
   * @codingStandardsIgnoreStart
   */
  public function _submitForm(array &$form, FormStateInterface $form_state) {
    //@codingStandardsIgnoreEnd
    $values = $form_state->getValues();
    if ($values['users_export_memory_limit'] != -1) {
      ini_set('memory_limit', $values['users_export_memory_limit']);
    }

    if ($values['users_export_max_execution'] != -1) {
      set_time_limit($values['users_export_max_execution']);
    }

    if ($values['users_export_filter_by_role'] == 1 && !empty($values['users_export_user_role'])) {
      $values['users_export_user_role'] = array_filter($values['users_export_user_role']);
    }
    else {
      $values['users_export_user_role'] = FALSE;
    }

    $exporters = $form_state->get('exporters');
    $exporter = $exporters[$values['users_export_type']]['class'];
    $exporter = new $exporter(new ExportData(), $values['users_export_filename']);

    $this->core->exporterLoadUsers($exporter, [
      'with_access' => $values['users_export_with_access'],
      'limit' => $values['users_export_test_mode'] ? 10 : NULL,
      'order' => $values['users_export_order'],
      'roles' => $values['users_export_user_role'],
      'status' => ($s = $values['users_export_user_status']) === 2 ? NULL : $s * 1,
      'date_format' => $values['users_export_date_format'],
    ]);

    $exporter->save();
  }

}
