<?php

namespace Drupal\users_export;

use AKlump\LoftDataGrids\ExportData;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Manager class to get available exporters.
 */
class UserExportLoftDataGrids {

  use StringTranslationTrait;

  /**
   * Returns the file_system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Returns the cache.default service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * Returns the current_user service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Returns the module_handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Construct UserExportLoftDataGrids object.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   Provides an interface for helpers that operate on files.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Defines an interface for cache implementations.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   Defines an interface for a service which has the current account stored.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   Interface for classes that manage a set of enabled modules.
   */
  public function __construct(FileSystemInterface $file_system, CacheBackendInterface $cache_backend, AccountProxyInterface $current_user, ModuleHandlerInterface $module_handler) {
    $this->fileSystem = $file_system;
    $this->cache = $cache_backend;
    $this->currentUser = $current_user;
    $this->moduleHandler = $module_handler;
  }

  /**
   * Return an options array of exporter classes available.
   *
   * @param bool $check_access
   *   Check to see if the current user has access to each option.
   * @param bool $include_description
   *   Check to include the description.
   * @param bool $use_shortname
   *   By default the label is used, set this to true to use the shortname.
   *
   * @return array
   *   Keys are classnames, values are human options.
   */
  public function getExporterOptions($check_access = TRUE, $include_description = TRUE, $use_shortname = TRUE) {
    $options = &drupal_static(__CLASS__ . '::' . __FUNCTION__, []);

    if (empty($options)) {
      foreach ($this->getExporters($check_access) as $info) {
        // Do not include html it will break for select lists.
        $label = $use_shortname && isset($info['shortname']) ? $info['shortname'] : $info['name'];
        $label .= ' (' . $info['extension'] . ')';
        if ($include_description) {
          $label .= ' ' . $info['description'];
        }
        $options[$info['id']] = $label;
      }
    }

    return $options;
  }

  /**
   * Return an array of all available exporters.
   *
   * @param bool $remove_no_access
   *   If $remove_no_access is true, those
   *   exporters which the current use does not
   *   have access to will be removed.
   *
   * @return array
   *   Each value is an array of data about the exporter with keys:
   */
  public function getExporters($remove_no_access = TRUE) {
    $exporters = [];
    $cid = 'loft_data_grids:exporters';
    if ($cache = $this->cache->get($cid)) {
      $exporters = $cache->data;
    }
    else {
      // @todo refactore the code.
//      $c = new \ReflectionClass(ExportData::class);
//      $path = dirname($c->getFileName());
//      $possible = $this->fileSystem->scanDirectory($path, '/.+Exporter\.php$/');
//      foreach ($possible as $path => $data) {
//        $class = '\\AKlump\\LoftDataGrids\\' . $data->name;
//
//        if (in_array('AKlump\LoftDataGrids\ExporterInterface', class_implements($class))) {
//          $obj = new \ReflectionClass($class);
//          if (!$obj->isAbstract()) {
//            $info = new $class(new ExportData());
//            $i = $info->getInfo();
//            $id = $this->generateExporterId($i);
//            $exporters[$id] = [
//                'access' => TRUE,
//              ] + $i;
//          }
//        }
//      }

      // Exporters added by our module.
      $info = new UsersExportDrupalTableExporter(new ExportData());
      $i = $info->getInfo();
      $exporters[$this->generateExporterId($i)] = $i;

      $this->cache->set($cid, $exporters);
    }

    // Add in the access flag for the current user.
    /** @var \Drupal\Core\Session\AccountProxyInterface $user */
    $user = $this->currentUser;
    array_walk($exporters, function (&$info) use ($user) {
      $perm = 'users_export:export as ' . $info['id'];
      $info['access'] = $user->hasPermission($perm);
    });

    $this->moduleHandler->alter('loft_data_grids_exporters', $exporters);

    if ($remove_no_access) {
      $exporters = array_filter($exporters, function ($value) {
        return $value['access'];
      });
    }

    return $exporters;
  }

  /**
   * Return the id for an exporter based on it's info array.
   */
  protected function generateExporterId(array &$info) {
    $id = md5($info['class']);
    $info = ['id' => $id] + $info;

    return $id;
  }

}
